package fe

import (
	"encoding/json"
	"log"
	"net"
	"testing"

	"git.autistici.org/ale/autoradio/Godeps/_workspace/src/github.com/miekg/dns"

	"git.autistici.org/ale/autoradio"
	"git.autistici.org/ale/autoradio/coordination/etcdtest"
)

func createTestDNSRedirector(t testing.TB, withNode bool) *DNSRedirector {
	etcd := etcdtest.NewClient()
	if withNode {
		// Create fake presence data, so that GetNodes() returns something.
		nodeData, _ := json.Marshal(&autoradio.NodeStatus{
			Name:      "node1",
			IcecastUp: true,
			IP: []net.IP{
				net.ParseIP("1.2.3.4"),
				net.ParseIP("2001:a:b::1"),
			},
		})
		log.Printf("creating %s", autoradio.NodePrefix+"0001")
		if _, err := etcd.Create(autoradio.NodePrefix+"0001", string(nodeData), 86400); err != nil {
			t.Fatalf("etcd.Create(): %v", err)
		}
	}

	client := autoradio.NewClient(etcd)
	client.WaitForNodes()
	d := NewDNSRedirector(client, "example.com", []net.IP{net.ParseIP("2.3.4.5")}, 30, []string{"ns1", "ns2"})
	d.updateEtcdCluster()
	return d
}

type testNetAddr struct{}

func (n testNetAddr) Network() string { return "tcp" }
func (n testNetAddr) String() string  { return "127.0.0.1" }

// Fake dns.ResponseWriter that records messages written to it.
type testDNSResponseWriter struct {
	messages []*dns.Msg
}

// WriteMsg writes a reply back to the client.
func (w *testDNSResponseWriter) WriteMsg(m *dns.Msg) error {
	w.messages = append(w.messages, m)
	return nil
}

// LocalAddr returns the net.Addr of the server
func (w *testDNSResponseWriter) LocalAddr() net.Addr { return &testNetAddr{} }

// RemoteAddr returns the net.Addr of the client that sent the current request.
func (w *testDNSResponseWriter) RemoteAddr() net.Addr { return &testNetAddr{} }

// Write writes a raw buffer back to the client.
func (w *testDNSResponseWriter) Write([]byte) (int, error) { return 0, nil }

// Close closes the connection.
func (w *testDNSResponseWriter) Close() error { return nil }

// TsigStatus returns the status of the Tsig.
func (w *testDNSResponseWriter) TsigStatus() error { return nil }

// TsigTimersOnly sets the tsig timers only boolean.
func (w *testDNSResponseWriter) TsigTimersOnly(bool) {}

// Hijack lets the caller take over the connection.
// After a call to Hijack(), the DNS package will not do anything with the connection.
func (w *testDNSResponseWriter) Hijack() {}

func TestDNSRedirector_A(t *testing.T) {
	testQueryA(t, true, "stream.example.com.", "1.2.3.4")
}

func TestDNSRedirector_A_Etcd(t *testing.T) {
	testQueryA(t, false, "etcd.example.com.", "127.0.0.1")
}

func TestDNSRedirector_A_Fallback(t *testing.T) {
	testQueryA(t, false, "stream.example.com.", "2.3.4.5")
}

func doTestQuery(t testing.TB, withNode bool, q *dns.Msg) *dns.Msg {
	d := createTestDNSRedirector(t, withNode)
	w := &testDNSResponseWriter{}
	d.ServeDNS(w, q)
	if len(w.messages) != 1 {
		t.Fatal("no reply")
	}
	return w.messages[0]
}

func testQueryA(t testing.TB, withNode bool, query, expected string) {
	q := new(dns.Msg)
	q.SetQuestion(query, dns.TypeA)
	response := doTestQuery(t, withNode, q)

	answer, ok := response.Answer[0].(*dns.A)
	if !ok {
		t.Fatalf("expected A, got: %s:", response.Answer[0])
	}
	if answer.A.String() != expected {
		t.Fatalf("bad IP: %s, expected %s", answer.A, expected)
	}
}

func TestDNSRedirector_AAAA(t *testing.T) {
	q := new(dns.Msg)
	q.SetQuestion("stream.example.com.", dns.TypeAAAA)
	response := doTestQuery(t, true, q)

	answer, ok := response.Answer[0].(*dns.AAAA)
	if !ok {
		t.Fatalf("bad reply (not AAAA): %s:", response.Answer[0])
	}
	expected := "2001:a:b::1"
	if answer.AAAA.String() != expected {
		t.Fatalf("bad IP: %s, expected %s", answer.AAAA, expected)
	}
}

func TestDNSRedirector_NXDOMAIN(t *testing.T) {
	q := new(dns.Msg)
	q.SetQuestion("nonexisting.example.com.", dns.TypeA)
	response := doTestQuery(t, false, q)
	if response.MsgHdr.Rcode != dns.RcodeNameError {
		t.Fatalf("expected NXDOMAIN, got: %s", response)
	}
}

func TestDNSRedirector_NXDOMAIN_WrongZone(t *testing.T) {
	q := new(dns.Msg)
	q.SetQuestion("foo.bar.com.", dns.TypeA)
	response := doTestQuery(t, false, q)
	if response.MsgHdr.Rcode != dns.RcodeNameError {
		t.Fatalf("expected NXDOMAIN, got: %s", response)
	}
}

func TestDNSRedirector_SRV_Etcd(t *testing.T) {
	q := new(dns.Msg)
	q.SetQuestion("_etcd-server._tcp.example.com.", dns.TypeSRV)
	response := doTestQuery(t, false, q)
	if len(response.Answer) != 1 {
		t.Fatalf("expected 1 answer, got: %s", response)
	}
	srv, ok := response.Answer[0].(*dns.SRV)
	if !ok {
		t.Fatalf("expected SRV, got: %s", response)
	}
	if srv.Port != 2379 {
		t.Fatalf("expected port 2379, got: %s", srv)
	}
	if srv.Target != "localhost." {
		t.Fatalf("expected target localhost, got: %s", srv)
	}
}

func TestDNSRedirector_SRV_Etcd_BadScheme(t *testing.T) {
	q := new(dns.Msg)
	q.SetQuestion("_etcd-server-ssl._tcp.example.com.", dns.TypeSRV)
	response := doTestQuery(t, false, q)
	if response.MsgHdr.Rcode != dns.RcodeNameError {
		t.Fatalf("expected NXDOMAIN, got: %s", response)
	}
}
